/********************************************************************************
*	NOTICE:														*
*																*
*	This code is believed by the author to be bug free.  You are free to use	*
*	and modify this code with the understanding that any use, either direct	*
*	or derivative, must contain acknowledgement of its author and source.		*
*	The author makes no warranty of any kind, expressed or implied, of		*
*	merchantability or fitness for a particular purpose.  The author shall	*
*	not be held liable for any incidental or consequential damages in		*
*	connection with or arising out of the furnishing, performance, or use of	*
*	this software.  This software is not authorized for use in life support	*
*	devices or systems.												*
*																*
********************************************************************************/

// AIPP software developed by Gianluca Di Pasqua, John Oh, and Sean Hanley
// in association with the DSP Lab at the University of Toronto, 2000.

import java.awt.*;
import java.awt.image.*;
import java.io.*;

// Directional-Distance Filter
public class DDF extends vectorFilter
{
	int indent;

	// Create object and initialize data
	public DDF(int size)
	{
		masksize = size;
		maskIndex = new int[masksize*masksize];
		array = new double[masksize*masksize];
		indent = (int) Math.floor(masksize/2);
	}

	// Runs the algorithm
	public void performEffect()
	{
		int counter;	
		int newPixels[] = new int [width*height];
		double temp_array[] = new double [masksize*masksize];

		// Moves through the input array pixel by pixel
		for (int y=indent;y<height-indent;y++)
		{
			for (int x=indent;x<width-indent;x++)
			{
				// Location of centre of mask
				int pointOffset = y*width+x;

				// Calculate Index for mask
				counter = 0;
				for (int i=-indent;i<=indent;i++)
				{
					for (int j=-indent;j<=indent;j++)
					{
						maskIndex[counter] = (y+i)*width+(x+j);
						counter++;
					}
				}

				// Distance from pixel to all others in mask
				calc_dist();
				
				// Place sum of distances in temp array
				for (int i=0; i<masksize*masksize; i++)
				{
					temp_array[i]=array[i];
				}
				
				// Angle from pixel to all others in mask
				calc_angle();
				
				// Calculate sum of dist mult by sum of angles
				for (int i=0; i<masksize*masksize; i++)
				{
					array[i]=array[i]*temp_array[i];
				}
				
				// Sort the new data array				
				b_sort();
				
				// Output smallest pixel value in array
				newPixels[pointOffset] = pixels[maskIndex[0]];
			}
		}
		this.pixels = newPixels;
	}
}
