/********************************************************************************
*	NOTICE:														*
*																*
*	This code is believed by the author to be bug free.  You are free to use	*
*	and modify this code with the understanding that any use, either direct	*
*	or derivative, must contain acknowledgement of its author and source.		*
*	The author makes no warranty of any kind, expressed or implied, of		*
*	merchantability or fitness for a particular purpose.  The author shall	*
*	not be held liable for any incidental or consequential damages in		*
*	connection with or arising out of the furnishing, performance, or use of	*
*	this software.  This software is not authorized for use in life support	*
*	devices or systems.												*
*																*
********************************************************************************/

// AIPP software developed by Gianluca Di Pasqua, John Oh, and Sean Hanley
// in association with the DSP Lab at the University of Toronto, 2000.

import java.awt.image.*;
import java.awt.*;

public abstract class EffectFilter extends ImageFilter
{
	int width;
	int height;
	int pixels[];

	public EffectFilter()
	{
	}

	public void setHints(int hints)
	{
		consumer.setHints(hints & ~ImageConsumer.COMPLETESCANLINES);
	}

	public void setDimensions(int width, int height)
	{
		this.width = width;
		this.height = height;
		this.pixels = new int[width*height];
		
		consumer.setDimensions(width, height);
	}

	public void setPixels(int x,int y,int width,int height,ColorModel model,byte[] pixels,int offset,int scansize)
	{
		for (int i=0;i<height;i++)
		{
			int destLineOffset = (y+i)*width;
			int srcLineOffset = i*scansize+offset;

			for (int j=0;j<width;j++)
			{
				int pixel = pixels[srcLineOffset+j]&0xff;
				this.pixels[destLineOffset+x+j]=model.getRGB(pixel);
			}
		}
	}
	
	public void setPixels(int x,int y,int width,int height,ColorModel model,int[] pixels,int offset,int scansize)
	{
		for (int i=0;i<height;i++)
		{
			int destLineOffset = (y+i)*width;
			int srcLineOffset = i*scansize+offset;

			for (int j=0;j<width;j++)
			{
				int pixel = pixels[srcLineOffset+j];
				this.pixels[destLineOffset+x+j]=model.getRGB(pixel);
			}
		}
	}

	public void imageComplete(int status)
	{
		performEffect();
		deliverPixels();
		super.imageComplete(status);
	}
	
	public abstract void performEffect();

	protected void deliverPixels()
	{
		consumer.setPixels(0,0,this.width,this.height,ColorModel.getRGBdefault(),this.pixels,0,this.width);
	}
}		
