/********************************************************************************
*	NOTICE:														*
*																*
*	This code is believed by the author to be bug free.  You are free to use	*
*	and modify this code with the understanding that any use, either direct	*
*	or derivative, must contain acknowledgement of its author and source.		*
*	The author makes no warranty of any kind, expressed or implied, of		*
*	merchantability or fitness for a particular purpose.  The author shall	*
*	not be held liable for any incidental or consequential damages in		*
*	connection with or arising out of the furnishing, performance, or use of	*
*	this software.  This software is not authorized for use in life support	*
*	devices or systems.												*
*																*
********************************************************************************/

// AIPP software developed by Gianluca Di Pasqua, John Oh, and Sean Hanley
// in association with the DSP Lab at the University of Toronto, 2000.

import java.awt.*;
import java.awt.image.*;
import java.io.*;

public class save_ppm extends EffectFilter
{
	String saveFilename;
	String saveFileDir;
	
	public save_ppm(String dir,String filename)
	{
		saveFileDir = dir;
		saveFilename = filename;
	}
	
	public void performEffect()
	{
		boolean imageIsGrey = true;
		String h1 = "P6";
		String h2 = "255";
		String h3 = "# Generated by AIPP, University of Toronto, DSP Lab";
		int new_line = 10; // ASCII value of CR 
		int blank_space = 32;

		String width_ascii, height_ascii;
		width_ascii = String.valueOf(width);
		height_ascii = String.valueOf(height);
		
		// parse image to check if image is greyscale
		// should use i++ if youreally want to be precise and check every pixel
		for(int i=0;i<width*height;i = i+(int)(width/10))
		{
			if (((pixels[i]>>16)&0xff) != ((pixels[i]>>8)&0xff)
				|| ((pixels[i]>>16)&0xff) != ((pixels[i])&0xff))
			{
				imageIsGrey = false;
				break;
			}
		}
		
		if (imageIsGrey == true)
		{
			// check for .pgm ext if not check add ext
			if (saveFilename.toLowerCase().endsWith(".pgm") == false)
			{
				saveFilename = saveFilename.substring(0,saveFilename.lastIndexOf("."));
				saveFilename = saveFilename.concat(".pgm");
			}
			// set header to type P5 for pgm
			h1 = "P5";
		}				
					
		try
		{
			FileWriter file = new FileWriter(saveFileDir+saveFilename);
			BufferedWriter file_save = new BufferedWriter(file);
			file_save.write(h1);
			file_save.write(new_line);
			file_save.write(h3);
			file_save.write(new_line);
			file_save.write(width_ascii);
			file_save.write(blank_space);
			file_save.write(height_ascii);
			file_save.write(new_line);
			file_save.write(h2);
			file_save.write(new_line);
						
			if (imageIsGrey)
			{
				// only write one value since they are all the same
				for (int x=0;x<width*height;x++)
				{
					file_save.write(((pixels[x]>>16)&0xff));
				}
			}
			else
			{
				for (int x=0;x<width*height;x++)
				{
					file_save.write(((pixels[x]>>16)&0xff));
					file_save.write(((pixels[x]>>8)&0xff));
					file_save.write(((pixels[x])&0xff));
				}
			}

			file_save.close();
			AIPP.statusBar.setText("Saved Image: "+saveFilename);
		}	
		catch (IOException se)
		{
			AIPP.statusBar.setText("Cannot save file: "+saveFilename);
		}					
	}
}
